{-# LANGUAGE ExistentialQuantification, DeriveDataTypeable, StandaloneDeriving #-}

module Phi.Phi ( Phi
               , Message
               , DefaultMessage(..)
               , fromMessage
               , initPhi
               , dupPhi
               , sendMessage
               , receiveMessage
               ) where

import Control.Concurrent.Chan
import Control.Monad
import Control.Monad.IO.Class

import Data.Typeable

data Phi = Phi (Chan Message)

data Message = forall a. (Typeable a, Show a) => Message a
deriving instance Show Message

data DefaultMessage = Repaint deriving (Typeable, Show)

fromMessage :: (Typeable a, Show a) => Message -> Maybe a
fromMessage (Message m) = cast m

initPhi :: MonadIO m => m Phi
initPhi = liftM Phi $ liftIO newChan

dupPhi :: MonadIO m => Phi -> m Phi
dupPhi (Phi chan) = liftM Phi $ liftIO $ dupChan chan

sendMessage :: (MonadIO m, Typeable a, Show a) => Phi -> a -> m ()
sendMessage (Phi chan) = liftIO . writeChan chan . Message

receiveMessage :: MonadIO m => Phi -> m Message
receiveMessage (Phi chan) = liftIO $ readChan chan
